@@ -1,5 +1,6 @@
 import React from "react";
 import { useCallback, useEffect, useState } from "react";
+import { ApplicationType } from "./constants";
 
 import {
     apiFetchLH2CalibrationState, apiApplyLH2Calibration,
@@ -13,6 +14,75 @@ const referencePoints = [
   {x: 0.1, y: -0.1},
 ]
 
+const DotBotsWaypoint = (props) => {
+  return (
+    <>
+      {(props.index === 0) ? (
+        <circle
+          cx={props.point.x * props.mapSize}
+          cy={props.point.y * props.mapSize}
+          r="4"
+          fill="none"
+          stroke={props.color}
+          strokeWidth="2"
+          opacity={props.opacity}
+        />
+      ) : (
+        <>
+          <line
+            x1={props.waypoints[props.index - 1].x * props.mapSize}
+            y1={props.waypoints[props.index - 1].y * props.mapSize}
+            x2={props.point.x * props.mapSize}
+            y2={props.point.y * props.mapSize}
+            stroke={props.color} strokeWidth="2" strokeDasharray="2" opacity={props.opacity}
+          />
+          <rect
+            x={props.point.x * props.mapSize - 2}
+            y={props.point.y * props.mapSize - 2}
+            width="4" height="4" fill={props.color} opacity={props.opacity}
+          />
+        </>
+      )}
+    </>
+  )
+}
+
+const DotBotsPosition = (props) => {
+  return (
+    <>
+      {(props.index === 0) ? (
+        <circle
+          cx={props.point.x * props.mapSize}
+          cy={props.point.y * props.mapSize}
+          r="4"
+          fill="none"
+          stroke={props.color}
+          strokeWidth="2"
+          opacity={props.opacity}
+        />
+      ) : (
+        <>
+          <line
+            x1={props.history[props.index - 1].x * props.mapSize}
+            y1={props.history[props.index - 1].y * props.mapSize}
+            x2={props.point.x * props.mapSize}
+            y2={props.point.y * props.mapSize}
+            stroke={props.color} strokeWidth="2"
+            opacity={props.opacity}
+          />
+          <circle
+            cx={props.point.x * props.mapSize}
+            cy={props.point.y * props.mapSize}
+            r="2"
+            fill={props.color}
+            opacity={props.opacity}
+          />
+        </>
+      )}
+    </>
+  )
+}
+
 const DotBotsMapPoint = (props) => {
   const [hovered, setHovered ] = useState(false);
 
@@ -23,6 +93,12 @@ const DotBotsMapPoint = (props) => {
 
   const posX = props.mapSize * parseFloat(props.dotbot.lh2_position.x);
   const posY = props.mapSize * parseFloat(props.dotbot.lh2_position.y);
+  const rotation = (props.dotbot.direction) ? props.dotbot.direction : 0;
+  const radius = (props.dotbot.address === props.active || hovered) ? 8: 5;
+  const directionShift = (props.dotbot.address === props.active || hovered) ? 2: 1;
+  const directionSize = (props.dotbot.address === props.active || hovered) ? 8: 5;
+  const opacity = `${props.dotbot.status === 0 ? "80%" : "20%"}`
+  const waypointOpacity = `${props.dotbot.status === 0 ? "50%" : "10%"}`
 
   const onMouseEnter = () => {
     if (props.dotbot.status !== 0) {
@@ -38,12 +114,20 @@ const DotBotsMapPoint = (props) => {
 
   return (
     <>
-    { (props.dotbot.address === props.active) &&
-      <circle cx={posX} cy={posY} r="8" stroke="black" strokeWidth="2" fill="none" />
-    }
+    {(props.dotbot.mode === 1 && props.dotbot.waypoints.length > 0) && (
+      props.dotbot.waypoints.map((point, index) => (
+        <DotBotsWaypoint key={`waypoint-${index}`} index={index} point={point} color={rgbColor} opacity={waypointOpacity} waypoints={props.dotbot.waypoints} {...props} />
+      ))
+    )}
+    {(props.showHistory && props.dotbot.position_history.length > 0) && (
+      props.dotbot.position_history.map((point, index) => (
+        <DotBotsPosition key={`position-${index}`} index={index} point={point} color={rgbColor} opacity={opacity} history={props.dotbot.position_history} {...props} />
+      ))
+    )}
+    <g transform={`rotate(${rotation} ${posX} ${posY})`} stroke={`${(props.dotbot.address === props.active) ? "black" : "none"}`} strokeWidth="1">
     <circle cx={posX} cy={posY}
-        r={(props.dotbot.address === props.active || hovered) ? 8: 5}
-        opacity={`${props.dotbot.status === 0 ? "80%" : "20%"}`}
+        r={radius}
+        opacity={opacity}
         fill={rgbColor}
         style={{ cursor: "pointer" }}
         onClick={
@@ -55,6 +139,8 @@ const DotBotsMapPoint = (props) => {
         onMouseLeave={onMouseLeave} >
       <title>{`${props.dotbot.address}@${posX}x${posY}`}</title>
     </circle>
+    {(props.dotbot.direction) && <polygon points={`${posX - radius + 2},${posY + radius + directionShift} ${posX + radius - 2},${posY + radius + directionShift} ${posX},${posY + radius + directionSize + directionShift}`} fill={rgbColor} opacity={opacity} />}
+    </g>
     </>
   )
 }
@@ -90,6 +176,14 @@ export const DotBotsMap = (props) => {
     }
   };
 
+  const mapClicked = (event) => {
+    const { farthestViewportElement: svgRoot } = event.target;
+    const dim = svgRoot.getBoundingClientRect();
+    const x = event.clientX - dim.left;
+    const y = event.clientY - dim.top;
+    props.mapClicked(x / props.mapSize, y / props.mapSize);
+  };
+
   const coordinateToPixel = (coordinate) => {
     return mapSize * (coordinate + 0.5) - 5;
   };
@@ -139,13 +233,13 @@ export const DotBotsMap = (props) => {
                 </pattern>
               </defs>
               {/* Map grid */}
-              <rect width="100%" height="100%" fill={displayGrid ? `url(#grid${mapSize})`: "none"} stroke="gray" strokeWidth="1"/>
+              <rect width="100%" height="100%" fill={displayGrid ? `url(#grid${mapSize})`: "none"} stroke="gray" strokeWidth="1" onClick={(event) => mapClicked(event)}/>
               {/* DotBots points */}
               {
                 props.dotbots && props.dotbots
-                  .filter(dotbot => dotbot.lh2_position)
                   .filter(dotbot => dotbot.status !== 2)
-                  .map(dotbot => <DotBotsMapPoint key={dotbot.address} dotbot={dotbot} active={props.active} updateActive={props.updateActive} mapSize={props.mapSize} />)
+                  .filter(dotbot => dotbot.lh2_position)
+                  .map(dotbot => <DotBotsMapPoint key={dotbot.address} dotbot={dotbot} active={props.active} updateActive={props.updateActive} showHistory={props.showHistory} mapSize={props.mapSize} />)
               }
               {
                 ["running", "ready"].includes(calibrationState) && (
@@ -169,6 +263,12 @@ export const DotBotsMap = (props) => {
               <label className="form-check-label" htmlFor="flexCheckDisplayGrid">Display grid</label>
             </div>
           </div>
+          <div className="d-flex mb-2">
+            <div className="form-check">
+              <input className="form-check-input" type="checkbox" id="flexCheckDisplayHistory" defaultChecked={props.showHistory} onChange={(event) => props.updateShowHistory(event.target.checked, ApplicationType.DotBot)} />
+              <label className="form-check-label" htmlFor="flexCheckDisplayHistory">Display position history</label>
+            </div>
+          </div>
           <div className="d-flex">
             <button className={`btn btn-sm ${calibrationButtonClass}`} onClick={calibrateClicked}>{calibrationButtonLabel}</button>
           </div>
