@@ -1,21 +1,25 @@
 """Module for the web server application."""
-
 import asyncio
 import os
 from binascii import hexlify
-from typing import List
+from typing import List, Union
 
 import uvicorn
-from fastapi import FastAPI, HTTPException, WebSocket, WebSocketDisconnect
+from fastapi import FastAPI, HTTPException, WebSocket, WebSocketDisconnect, Depends
 from fastapi.middleware.cors import CORSMiddleware
 from fastapi.staticfiles import StaticFiles
 
+from dotbot import pydotbot_version
 from dotbot.models import (
     DotBotCalibrationStateModel,
     DotBotModel,
+    DotBotQueryModel,
     DotBotAddressModel,
     DotBotMoveRawCommandModel,
     DotBotRgbLedCommandModel,
+    DotBotControlModeModel,
+    DotBotLH2Position,
+    DotBotGPSPosition,
 )
 from dotbot.protocol import (
     PROTOCOL_VERSION,
@@ -24,6 +28,11 @@ from dotbot.protocol import (
     PayloadType,
     CommandMoveRaw,
     CommandRgbLed,
+    ControlMode,
+    LH2Location,
+    LH2Waypoints,
+    GPSPosition,
+    GPSWaypoints,
     ApplicationType,
 )
 
@@ -35,7 +44,7 @@ app = FastAPI(
     debug=0,
     title="DotBot controller API",
     description="This is the DotBot controller API",
-    version="1.0.0",
+    version=pydotbot_version(),
     docs_url="/api",
     redoc_url=None,
 )
@@ -134,6 +143,86 @@ async def dotbots_rgb_led(
     app.controller.dotbots[address].rgb_led = command
 
 
+@app.put(
+    path="/controller/dotbots/{address}/{application}/mode",
+    summary="Set the dotbot control mode",
+    tags=["dotbots"],
+)
+async def dotbots_mode(address: str, application: int, data: DotBotControlModeModel):
+    """Set the control mode of a DotBot."""
+    if address not in app.controller.dotbots:
+        raise HTTPException(status_code=404, detail="No matching dotbot found")
+
+    header = ProtocolHeader(
+        destination=int(address, 16),
+        source=int(app.controller.settings.gw_address, 16),
+        swarm_id=int(app.controller.settings.swarm_id, 16),
+        application=ApplicationType(application),
+        version=PROTOCOL_VERSION,
+    )
+    payload = ProtocolPayload(
+        header,
+        PayloadType.CONTROL_MODE,
+        ControlMode(data.mode),
+    )
+    app.controller.send_payload(payload)
+    app.controller.dotbots[address].mode = data.mode
+
+
+@app.put(
+    path="/controller/dotbots/{address}/{application}/waypoints",
+    summary="Set the dotbot control mode",
+    tags=["dotbots"],
+)
+async def dotbots_waypoints(
+    address: str,
+    application: int,
+    waypoints: List[Union[DotBotLH2Position, DotBotGPSPosition]],
+):
+    """Set the lh2 waypoints of a DotBot."""
+    if address not in app.controller.dotbots:
+        raise HTTPException(status_code=404, detail="No matching dotbot found")
+
+    header = ProtocolHeader(
+        destination=int(address, 16),
+        source=int(app.controller.settings.gw_address, 16),
+        swarm_id=int(app.controller.settings.swarm_id, 16),
+        application=ApplicationType(application),
+        version=PROTOCOL_VERSION,
+    )
+    if ApplicationType(application) == ApplicationType.SailBot:
+        payload = ProtocolPayload(
+            header,
+            PayloadType.GPS_WAYPOINTS,
+            GPSWaypoints(
+                [
+                    GPSPosition(
+                        latitude=int(waypoint.latitude * 1e6),
+                        longitude=int(waypoint.longitude * 1e6),
+                    )
+                    for waypoint in waypoints
+                ]
+            ),
+        )
+    else:  # DotBot application
+        payload = ProtocolPayload(
+            header,
+            PayloadType.LH2_WAYPOINTS,
+            LH2Waypoints(
+                [
+                    LH2Location(
+                        pos_x=int(waypoint.x * 1e6),
+                        pos_y=int(waypoint.y * 1e6),
+                        pos_z=int(waypoint.z * 1e6),
+                    )
+                    for waypoint in waypoints
+                ]
+            ),
+        )
+    app.controller.dotbots[address].waypoints = waypoints
+    app.controller.send_payload(payload)
+
+
 @app.get(
     path="/controller/dotbots/{address}",
     response_model=DotBotModel,
@@ -141,11 +230,13 @@ async def dotbots_rgb_led(
     summary="Return information about a dotbot given its address",
     tags=["dotbots"],
 )
-async def dotbot(address: str):
+async def dotbot(address: str, query: DotBotQueryModel = Depends()):
     """Dotbot HTTP GET handler."""
     if address not in app.controller.dotbots:
         raise HTTPException(status_code=404, detail="No matching dotbot found")
-    return app.controller.dotbots[address]
+    _dotbot = DotBotModel(**app.controller.dotbots[address].dict())
+    _dotbot.position_history = _dotbot.position_history[: query.max_positions]
+    return _dotbot
 
 
 @app.get(
@@ -155,11 +246,9 @@ async def dotbot(address: str):
     summary="Return the list of available dotbots",
     tags=["dotbots"],
 )
-async def dotbots():
+async def dotbots(query: DotBotQueryModel = Depends()):
     """Dotbots HTTP GET handler."""
-    return sorted(
-        list(app.controller.dotbots.values()), key=lambda dotbot: dotbot.address
-    )
+    return app.controller.get_dotbots(query)
 
 
 @app.post(
